@@ -12,23 +12,39 @@ import (
 	"log"
 	"math/rand"
 	"os"
+	"path"
 	"path/filepath"
 	"regexp"
+	"runtime"
 	"sort"
 	"strings"
 	"testing"
 	"time"
 
 	"github.com/ncw/rclone/fs"
+	"github.com/ncw/rclone/fs/accounting"
+	"github.com/ncw/rclone/fs/config"
+	"github.com/ncw/rclone/fs/hash"
+	"github.com/ncw/rclone/fs/walk"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"golang.org/x/text/unicode/norm"
 )
 
+// Globals
 var (
-	// MatchTestRemote matches the remote names used for testing
-	MatchTestRemote = regexp.MustCompile(`^rclone-test-[abcdefghijklmnopqrstuvwxyz0123456789]{24}$`)
+	RemoteName      = flag.String("remote", "", "Remote to test with, defaults to local filesystem")
+	SubDir          = flag.Bool("subdir", false, "Set to test with a sub directory")
+	Verbose         = flag.Bool("verbose", false, "Set to enable logging")
+	DumpHeaders     = flag.Bool("dump-headers", false, "Set to dump headers (needs -verbose)")
+	DumpBodies      = flag.Bool("dump-bodies", false, "Set to dump bodies (needs -verbose)")
+	Individual      = flag.Bool("individual", false, "Make individual bucket/container/directory for each test - much slower")
+	LowLevelRetries = flag.Int("low-level-retries", 10, "Number of low level retries")
+	UseListR        = flag.Bool("fast-list", false, "Use recursive list if available. Uses more memory but fewer transactions.")
 	// ListRetries is the number of times to retry a listing to overcome eventual consistency
 	ListRetries = flag.Int("list-retries", 6, "Number or times to retry listing")
+	// MatchTestRemote matches the remote names used for testing
+	MatchTestRemote = regexp.MustCompile(`^rclone-test-[abcdefghijklmnopqrstuvwxyz0123456789]{24}$`)
 )
 
 // Seed the random number generator
@@ -37,10 +53,36 @@ func init() {
 
 }
 
+// Initialise rclone for testing
+func Initialise() {
+	// Never ask for passwords, fail instead.
+	// If your local config is encrypted set environment variable
+	// "RCLONE_CONFIG_PASS=hunter2" (or your password)
+	fs.Config.AskPassword = false
+	// Override the config file from the environment - we don't
+	// parse the flags any more so this doesn't happen
+	// automatically
+	if envConfig := os.Getenv("RCLONE_CONFIG"); envConfig != "" {
+		config.ConfigPath = envConfig
+	}
+	config.LoadConfig()
+	if *Verbose {
+		fs.Config.LogLevel = fs.LogLevelDebug
+	}
+	if *DumpHeaders {
+		fs.Config.Dump |= fs.DumpHeaders
+	}
+	if *DumpBodies {
+		fs.Config.Dump |= fs.DumpBodies
+	}
+	fs.Config.LowLevelRetries = *LowLevelRetries
+	fs.Config.UseListR = *UseListR
+}
+
 // Item represents an item for checking
 type Item struct {
 	Path    string
-	Hashes  map[fs.HashType]string
+	Hashes  map[hash.Type]string
 	ModTime time.Time
 	Size    int64
 	WinPath string
@@ -53,7 +95,7 @@ func NewItem(Path, Content string, modTime time.Time) Item {
 		ModTime: modTime,
 		Size:    int64(len(Content)),
 	}
-	hash := fs.NewMultiHasher()
+	hash := hash.NewMultiHasher()
 	buf := bytes.NewBufferString(Content)
 	_, err := io.Copy(hash, buf)
 	if err != nil {
@@ -83,21 +125,42 @@ func (i *Item) CheckModTime(t *testing.T, obj fs.Object, modTime time.Time, prec
 func (i *Item) CheckHashes(t *testing.T, obj fs.Object) {
 	require.NotNil(t, obj)
 	types := obj.Fs().Hashes().Array()
-	for _, hash := range types {
+	for _, Hash := range types {
 		// Check attributes
-		sum, err := obj.Hash(hash)
+		sum, err := obj.Hash(Hash)
 		require.NoError(t, err)
-		assert.True(t, fs.HashEquals(i.Hashes[hash], sum), fmt.Sprintf("%s/%s: %v hash incorrect - expecting %q got %q", obj.Fs().String(), obj.Remote(), hash, i.Hashes[hash], sum))
+		assert.True(t, hash.Equals(i.Hashes[Hash], sum), fmt.Sprintf("%s/%s: %v hash incorrect - expecting %q got %q", obj.Fs().String(), obj.Remote(), Hash, i.Hashes[Hash], sum))
 	}
 }
 
 // Check checks all the attributes of the object are correct
 func (i *Item) Check(t *testing.T, obj fs.Object, precision time.Duration) {
 	i.CheckHashes(t, obj)
-	assert.Equal(t, i.Size, obj.Size(), fmt.Sprintf("%s: size incorrect", i.Path))
+	assert.Equal(t, i.Size, obj.Size(), fmt.Sprintf("%s: size incorrect file=%d vs obj=%d", i.Path, i.Size, obj.Size()))
 	i.CheckModTime(t, obj, obj.ModTime(), precision)
 }
 
+// WinPath converts a path into a windows safe path
+func WinPath(s string) string {
+	return strings.Map(func(r rune) rune {
+		switch r {
+		case '<', '>', '"', '|', '?', '*', ':':
+			return '_'
+		}
+		return r
+	}, s)
+}
+
+// Normalize runs a utf8 normalization on the string if running on OS
+// X.  This is because OS X denormalizes file names it writes to the
+// local file system.
+func Normalize(name string) string {
+	if runtime.GOOS == "darwin" {
+		name = norm.NFC.String(name)
+	}
+	return name
+}
+
 // Items represents all items for checking
 type Items struct {
 	byName    map[string]*Item
@@ -114,18 +177,19 @@ func NewItems(items []Item) *Items {
 	}
 	// Fill up byName
 	for i := range items {
-		is.byName[items[i].Path] = &items[i]
-		is.byNameAlt[items[i].WinPath] = &items[i]
+		is.byName[Normalize(items[i].Path)] = &items[i]
+		is.byNameAlt[Normalize(items[i].WinPath)] = &items[i]
 	}
 	return is
 }
 
 // Find checks off an item
 func (is *Items) Find(t *testing.T, obj fs.Object, precision time.Duration) {
-	i, ok := is.byName[obj.Remote()]
+	remote := Normalize(obj.Remote())
+	i, ok := is.byName[remote]
 	if !ok {
-		i, ok = is.byNameAlt[obj.Remote()]
-		assert.True(t, ok, fmt.Sprintf("Unexpected file %q", obj.Remote()))
+		i, ok = is.byNameAlt[remote]
+		assert.True(t, ok, fmt.Sprintf("Unexpected file %q", remote))
 	}
 	if i != nil {
 		delete(is.byName, i.Path)
@@ -151,8 +215,8 @@ func makeListingFromItems(items []Item) (string, string) {
 	nameLengths1 := make([]string, len(items))
 	nameLengths2 := make([]string, len(items))
 	for i, item := range items {
-		remote1 := item.Path
-		remote2 := item.Path
+		remote1 := Normalize(item.Path)
+		remote2 := remote1
 		if item.WinPath != "" {
 			remote2 = item.WinPath
 		}
@@ -168,12 +232,36 @@ func makeListingFromItems(items []Item) (string, string) {
 func makeListingFromObjects(objs []fs.Object) string {
 	nameLengths := make([]string, len(objs))
 	for i, obj := range objs {
-		nameLengths[i] = fmt.Sprintf("%s (%d)", obj.Remote(), obj.Size())
+		nameLengths[i] = fmt.Sprintf("%s (%d)", Normalize(obj.Remote()), obj.Size())
 	}
 	sort.Strings(nameLengths)
 	return strings.Join(nameLengths, ", ")
 }
 
+// filterEmptyDirs removes any empty (or containing only directories)
+// directories from expectedDirs
+func filterEmptyDirs(t *testing.T, items []Item, expectedDirs []string) (newExpectedDirs []string) {
+	dirs := map[string]struct{}{"": struct{}{}}
+	for _, item := range items {
+		base := item.Path
+		for {
+			base = path.Dir(base)
+			if base == "." || base == "/" {
+				break
+			}
+			dirs[base] = struct{}{}
+		}
+	}
+	for _, expectedDir := range expectedDirs {
+		if _, found := dirs[expectedDir]; found {
+			newExpectedDirs = append(newExpectedDirs, expectedDir)
+		} else {
+			t.Logf("Filtering empty directory %q", expectedDir)
+		}
+	}
+	return newExpectedDirs
+}
+
 // CheckListingWithPrecision checks the fs to see if it has the
 // expected contents with the given precision.
 //
@@ -181,10 +269,13 @@ func makeListingFromObjects(objs []fs.Object) string {
 // directories returned is also OK as some remotes don't return
 // directories.
 func CheckListingWithPrecision(t *testing.T, f fs.Fs, items []Item, expectedDirs []string, precision time.Duration) {
+	if expectedDirs != nil && !f.Features().CanHaveEmptyDirectories {
+		expectedDirs = filterEmptyDirs(t, items, expectedDirs)
+	}
 	is := NewItems(items)
-	oldErrors := fs.Stats.GetErrors()
+	oldErrors := accounting.Stats.GetErrors()
 	var objs []fs.Object
-	var dirs []*fs.Dir
+	var dirs []fs.Directory
 	var err error
 	var retries = *ListRetries
 	sleep := time.Second / 2
@@ -192,13 +283,14 @@ func CheckListingWithPrecision(t *testing.T, f fs.Fs, items []Item, expectedDirs
 	gotListing := "<unset>"
 	listingOK := false
 	for i := 1; i <= retries; i++ {
-		objs, dirs, err = fs.NewLister().Start(f, "").GetAll()
+		objs, dirs, err = walk.GetAll(f, "", true, -1)
 		if err != nil && err != fs.ErrorDirNotFound {
 			t.Fatalf("Error listing: %v", err)
 		}
+
 		gotListing = makeListingFromObjects(objs)
 		listingOK = wantListing1 == gotListing || wantListing2 == gotListing
-		if listingOK && (expectedDirs == nil || len(dirs) == 0 || len(dirs) == len(expectedDirs)) {
+		if listingOK && (expectedDirs == nil || len(dirs) == len(expectedDirs)) {
 			// Put an extra sleep in if we did any retries just to make sure it really
 			// is consistent (here is looking at you Amazon Drive!)
 			if i != 1 {
@@ -223,19 +315,22 @@ func CheckListingWithPrecision(t *testing.T, f fs.Fs, items []Item, expectedDirs
 	}
 	is.Done(t)
 	// Don't notice an error when listing an empty directory
-	if len(items) == 0 && oldErrors == 0 && fs.Stats.GetErrors() == 1 {
-		fs.Stats.ResetErrors()
+	if len(items) == 0 && oldErrors == 0 && accounting.Stats.GetErrors() == 1 {
+		accounting.Stats.ResetErrors()
 	}
-	// Check the directories - ignore if no directories returned
-	// for remotes which can't do directories
-	if expectedDirs != nil && len(dirs) != 0 {
+	// Check the directories
+	if expectedDirs != nil {
+		expectedDirsCopy := make([]string, len(expectedDirs))
+		for i, dir := range expectedDirs {
+			expectedDirsCopy[i] = WinPath(Normalize(dir))
+		}
 		actualDirs := []string{}
 		for _, dir := range dirs {
-			actualDirs = append(actualDirs, dir.Name)
+			actualDirs = append(actualDirs, WinPath(Normalize(dir.Remote())))
 		}
 		sort.Strings(actualDirs)
-		sort.Strings(expectedDirs)
-		assert.Equal(t, expectedDirs, actualDirs, "directories")
+		sort.Strings(expectedDirsCopy)
+		assert.Equal(t, expectedDirsCopy, actualDirs, "directories")
 	}
 }
 
@@ -344,9 +439,9 @@ func RandomRemote(remoteName string, subdir bool) (fs.Fs, string, func(), error)
 	}
 
 	finalise := func() {
-		_ = fs.Purge(remote) // ignore error
+		Purge(remote)
 		if parentRemote != nil {
-			err = fs.Purge(parentRemote) // ignore error
+			Purge(parentRemote)
 			if err != nil {
 				log.Printf("Failed to purge %v: %v", parentRemote, err)
 			}
@@ -356,22 +451,51 @@ func RandomRemote(remoteName string, subdir bool) (fs.Fs, string, func(), error)
 	return remote, remoteName, finalise, nil
 }
 
-// TestMkdir tests Mkdir works
-func TestMkdir(t *testing.T, remote fs.Fs) {
-	err := fs.Mkdir(remote, "")
-	require.NoError(t, err)
-	CheckListing(t, remote, []Item{})
-}
-
-// TestPurge tests Purge works
-func TestPurge(t *testing.T, remote fs.Fs) {
-	err := fs.Purge(remote)
-	require.NoError(t, err)
-	CheckListing(t, remote, []Item{})
-}
-
-// TestRmdir tests Rmdir works
-func TestRmdir(t *testing.T, remote fs.Fs) {
-	err := fs.Rmdir(remote, "")
-	require.NoError(t, err)
+// Purge is a simplified re-implementation of operations.Purge for the
+// test routine cleanup to avoid circular dependencies.
+//
+// It logs errors rather than returning them
+func Purge(f fs.Fs) {
+	var err error
+	doFallbackPurge := true
+	if doPurge := f.Features().Purge; doPurge != nil {
+		doFallbackPurge = false
+		fs.Debugf(f, "Purge remote")
+		err = doPurge()
+		if err == fs.ErrorCantPurge {
+			doFallbackPurge = true
+		}
+	}
+	if doFallbackPurge {
+		dirs := []string{""}
+		err = walk.Walk(f, "", true, -1, func(dirPath string, entries fs.DirEntries, err error) error {
+			if err != nil {
+				log.Printf("purge walk returned error: %v", err)
+				return nil
+			}
+			entries.ForObject(func(obj fs.Object) {
+				fs.Debugf(f, "Purge object %q", obj.Remote())
+				err = obj.Remove()
+				if err != nil {
+					log.Printf("purge failed to remove %q: %v", obj.Remote(), err)
+				}
+			})
+			entries.ForDir(func(dir fs.Directory) {
+				dirs = append(dirs, dir.Remote())
+			})
+			return nil
+		})
+		sort.Strings(dirs)
+		for i := len(dirs) - 1; i >= 0; i-- {
+			dir := dirs[i]
+			fs.Debugf(f, "Purge dir %q", dir)
+			err := f.Rmdir(dir)
+			if err != nil {
+				log.Printf("purge failed to rmdir %q: %v", dir, err)
+			}
+		}
+	}
+	if err != nil {
+		log.Printf("purge failed: %v", err)
+	}
 }
